#######################################################################
#
# NCM component for profile daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.2 : 11/01/05 18:24
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::profile;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;
use File::Copy;

use EDG::WP4::CCM::Element;

use File::Path;
use File::Basename;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/profile";

    # Get the configuration directory.
    my $cfgdir = "/etc/profile.d";
    if ($config->elementExists("$base/configDir")) {
        $cfgdir = $config->getValue("$base/configDir");
    }

    # Ensure that the configuration directory exists. 
    mkpath($cfgdir,0,0755) if (! -e $cfgdir);
    if (! -d $cfgdir) {
	$self->error("Cannot create $cfgdir or it isn't a directory");
	return 1;
    }

    # Get the configuration file name (without suffix).
    my $cfgname = "env";
    if ($config->elementExists("$base/configName")) {
        $cfgname = $config->getValue("$base/configName");
    }

    # Initialize the contents of the scripts. 
    my $comment = "#\n# Created by ncm-profile on " . localtime() . "\n" .  
	          "# Manual changes will be overwritten.\n#\n";
    my $sh = $comment;
    my $csh = $comment;

    # First loop over all of the environment variables. 
    if ($config->elementExists("$base/env")) {
	my $root = $config->getElement("$base/env");
	while ($root->hasNextElement()) {
	    my $element = $root->getNextElement();
	    if ($element->isProperty()) {
		my $key = $element->getName();
		my $value = $element->getValue();
		
		$sh .= "export $key=\"" . escape_sh($value) . "\"\n";
		$csh .= "setenv $key \"". escape_csh($value) . "\"\n";
	    }
	}
    }

    # Now loop over all of the paths. 
    if ($config->elementExists("$base/path")) {
	my $root = $config->getElement("$base/path");
	while ($root->hasNextElement()) {
	    my $element = $root->getNextElement();
	    if ($element->isResource()) {
		my $key = $element->getName();

		my $setPath = 0;

		# A hash of existing elements.  This is used to avoid duplicates. 
		my %epaths;

		# Do the prepended values first.  This make duplicate
		# removal easier later.
		my @pre;
		if ($config->elementExists("$base/path/$key/prepend")) {
		    my @list = $config->getElement("$base/path/$key/prepend")->getList();
		    foreach my $e (@list) {
			my $v = $e->getValue();
			push @pre, $v unless $epaths{$v};
			$epaths{$v} = 1;
		    }
		}

		# Now the definitive value if set.  Set flag for this
		# case. 
		my @path;
		if ($config->elementExists("$base/path/$key/value")) {
		    $setPath = 1;
		    my @list = $config->getElement("$base/path/$key/value")->getList();
		    foreach my $e (@list) {
			my $v = $e->getValue();
			push @path, $v unless $epaths{$v};
			$epaths{$v} = 1;
		    }
		}

		# Finally the appended values. 
		my @post;
		if ($config->elementExists("$base/path/$key/append")) {
		    my @list = $config->getElement("$base/path/$key/append")->getList();
		    foreach my $e (@list) {
			my $v = $e->getValue();
			push @post, $v unless $epaths{$v};
			$epaths{$v} = 1;
		    }
		}

		# Export a definitive value.
		if ($setPath) {
		    my $s = join(':',@path);
		    $s = '""' if ($s eq '');

		    $sh .= "export $key=\"$s\"\n";
		    $csh .= "setenv $key \"$s\"\n";
		}

		# Now add the prepended values.  Be careful to ensure
		# proper behavior if the variable is not defined. 
		if ($#pre >= 0) {
		    my $s = join(':',@pre);

		    my $ssh = escape_sh($s);
		    $sh .= "if [ -z \${$key} ]; then\n";
		    $sh .= "$key=\"$ssh\"\n";
		    $sh .= "else\n";
		    $sh .= "$key=\"$ssh:\${$key}\"\n";
		    $sh .= "fi\n";
		    $sh .= "export $key\n\n";

		    my $scsh = escape_csh($s);
		    $csh .= "if (\$?$key) then\n";
		    $csh .= "setenv $key \"$scsh:\${$key}\"\n";
		    $csh .= "else\n";
		    $csh .= "setenv $key \"$scsh\"\n";
		    $csh .= "endif\n\n";
		}

		# Now add the prepended values.  Be careful to ensure
		# proper behavior if the variable is not defined. 
		if ($#post >= 0) {
		    my $s = join(':',@post);

		    my $ssh = escape_sh($s);
		    $sh .= "if [ -z \${$key} ]; then\n";
		    $sh .= "$key=\"$ssh\"\n";
		    $sh .= "else\n";
		    $sh .= "$key=\"\${$key}:$ssh\"\n";
		    $sh .= "fi\n";
		    $sh .= "export $key\n\n";

		    my $scsh = escape_csh($s);
		    $csh .= "if (\$?$key) then\n";
		    $csh .= "setenv $key \"\${$key}:$scsh\"\n";
		    $csh .= "else\n";
		    $csh .= "setenv $key \"$scsh\"\n";
		    $csh .= "endif\n\n";
		}

	    }
	}
    }

    # Append the name of the component as a suffix so that old files
    # can be deleted.  
    $cfgname = $cfgname . ".ncm-profile";

    # Delete all of the existing profiles.
    opendir DIR, $cfgdir;
    my @scripts = grep /\.ncm-profile.c?sh/, readdir DIR;
    closedir DIR; 
    unlink @scripts;

    # Do the sh script.
    my $fname = $cfgdir . '/' . $cfgname . ".sh";
    my $result = LC::Check::file( $fname,
				  backup => ".old",
				  contents => $sh,
				  );
    $self->log("$fname created") if $result;
    chmod 0755, $fname;

    # Create the csh script. 
    $fname = $cfgdir . '/' . $cfgname . ".csh";
    $result = LC::Check::file( $fname,
			       backup => ".old",
			       contents => $csh,
			       );
    $self->log("$fname created") if $result;
    chmod 0755, $fname;

    return 1;
}

sub escape_sh {
    my ($s) = @_;
    $s =~ s/"/\\"/g;
    return $s;
}

sub escape_csh {
    my ($s) = @_;
    $s =~ s/"/"\\""/g;
    return $s;
}

1;      # Required for PERL modules
